import os
from pathlib import Path

import pytest
from pytest_cases import fixture_union


@pytest.fixture
def abs_path(tmp_path):
    """Fixture that returns a temporary directory as an absolute Path object."""
    return tmp_path


@pytest.fixture
def rel_path(tmp_path):
    """Fixture that sets a temporary directory as the current working directory and returns a
    relative path to it."""
    orig_wd = Path.cwd()
    os.chdir(tmp_path)
    yield Path()
    os.chdir(orig_wd)


base_path = fixture_union("base_path", ["rel_path", "abs_path"])



from zipfile import ZipFile, ZipInfo

from _typeshed import SizedBuffer, StrPath

__all__ = ["date_time", "file_mode", "dir_mode", "ReproducibleZipFile"]
__version__: str

def date_time() -> tuple[int, int, int, int, int, int]: ...
def file_mode() -> int: ...
def dir_mode() -> int: ...

class ReproducibleZipFile(ZipFile):
    def write(
        self,
        filename: StrPath,
        arcname: StrPath | None = None,
        compress_type: int | None = None,
        compresslevel: int | None = None,
    ) -> None: ...
    def writestr(
        self,
        zinfo_or_arcname: str | ZipInfo,
        data: SizedBuffer | str,
        compress_type: int | None = None,
        compresslevel: int | None = None,
    ) -> None: ...
    def mkdir(self, zinfo_or_directory_name: str | ZipInfo, mode: int = 511) -> None: ...



import platform
from time import sleep
from zipfile import ZipFile, ZipInfo

try:
    from time import tzset
except ImportError:
    tzset = None  # type: ignore[assignment]

from repro_zipfile import ReproducibleZipFile
from tests.utils import (
    assert_archive_contents_equals,
    data_factory,
    dir_tree_factory,
    file_factory,
    hash_file,
    umask,
)


def test_write_dir_tree_mtime(base_path):
    """Archiving a directory tree works with different modified time."""
    dir_tree = dir_tree_factory(base_path)

    # Create base ReproducibleZipFile archive
    repro_zipfile_arc1 = base_path / "repro_zipfile1.zip"
    with ReproducibleZipFile(repro_zipfile_arc1, "w") as zp:
        for path in sorted(dir_tree.glob("**/*")):
            zp.write(path)

    # Create regular ZipFile archive for comparison
    zipfile_arc1 = base_path / "zipfile1.zip"
    with ZipFile(zipfile_arc1, "w") as zp:
        for path in sorted(dir_tree.glob("**/*")):
            zp.write(path)

    # Sleep to update modified times, change permissions
    sleep(2)
    for path in dir_tree.glob("**/*"):
        path.touch()

    # Create second ReproducibleZipFile archive after delay
    repro_zipfile_arc2 = base_path / "repro_zipfile2.zip"
    with ReproducibleZipFile(repro_zipfile_arc2, "w") as zp:
        for path in sorted(dir_tree.glob("**/*")):
            zp.write(path)

    # Create second regular ZipFile archive for comparison after delay
    zipfile_arc2 = base_path / "zipfile2.zip"
    with ZipFile(zipfile_arc2, "w") as zp:
        for path in sorted(dir_tree.glob("**/*")):
            zp.write(path)

    # All four archives should have identical content
    assert_archive_contents_equals(repro_zipfile_arc1, zipfile_arc1)
    assert_archive_contents_equals(repro_zipfile_arc1, repro_zipfile_arc2)
    assert_archive_contents_equals(repro_zipfile_arc1, zipfile_arc2)

    # ReproducibleZipFile hashes should match; ZipFile hashes should not
    assert hash_file(repro_zipfile_arc1) == hash_file(repro_zipfile_arc2)
    assert hash_file(zipfile_arc1) != hash_file(zipfile_arc2)


def test_write_dir_tree_mode(base_path):
    """Archiving a directory tree works with different permission modes."""
    with umask(0o022):
        dir_tree = dir_tree_factory(base_path)

    # Create base ReproducibleZipFile archive
    repro_zipfile_arc1 = base_path / "repro_zipfile1.zip"
    with ReproducibleZipFile(repro_zipfile_arc1, "w") as zp:
        for path in sorted(dir_tree.glob("**/*")):
            zp.write(path)

    # Create regular ZipFile archive for comparison
    zipfile_arc1 = base_path / "zipfile1.zip"
    with ZipFile(zipfile_arc1, "w") as zp:
        for path in sorted(dir_tree.glob("**/*")):
            zp.write(path)

    # Change permissions
    with umask(0o002) as mask:
        dir_tree.chmod(mode=0o777 ^ mask)
        for path in dir_tree.glob("**/*"):
            if path.is_file():
                path.chmod(mode=0o666 ^ mask)
            else:
                path.chmod(mode=0o777 ^ mask)

    # Create second ReproducibleZipFile archive after delay
    repro_zipfile_arc2 = base_path / "repro_zipfile2.zip"
    with ReproducibleZipFile(repro_zipfile_arc2, "w") as zp:
        for path in sorted(dir_tree.glob("**/*")):
            zp.write(path)

    # Create second regular ZipFile archive for comparison after delay
    zipfile_arc2 = base_path / "zipfile2.zip"
    with ZipFile(zipfile_arc2, "w") as zp:
        for path in sorted(dir_tree.glob("**/*")):
            zp.write(path)

    # All four archives should have identical content
    assert_archive_contents_equals(repro_zipfile_arc1, zipfile_arc1)
    assert_archive_contents_equals(repro_zipfile_arc1, repro_zipfile_arc2)
    assert_archive_contents_equals(repro_zipfile_arc1, zipfile_arc2)

    # ReproducibleZipFile hashes should match; ZipFile hashes should not
    assert hash_file(repro_zipfile_arc1) == hash_file(repro_zipfile_arc2)
    if platform.system() != "Windows":
        # Windows doesn't seem to actually make them different
        assert hash_file(zipfile_arc1) != hash_file(zipfile_arc2)


def test_write_dir_tree_string_paths(rel_path):
    """Archiving a directory tree works."""
    dir_tree = dir_tree_factory(rel_path)

    # Create base ReproducibleZipFile archive
    repro_zipfile_arc1 = rel_path / "repro_zipfile1.zip"
    with ReproducibleZipFile(repro_zipfile_arc1, "w") as zp:
        for path in sorted(dir_tree.glob("**/*")):
            zp.write(str(path))

    # Create regular ZipFile archive for comparison
    zipfile_arc1 = rel_path / "zipfile1.zip"
    with ZipFile(zipfile_arc1, "w") as zp:
        for path in sorted(dir_tree.glob("**/*")):
            zp.write(str(path))

    # Update modified times
    sleep(2)
    for path in dir_tree.glob("**/*"):
        path.touch()

    # Create second ReproducibleZipFile archive after delay
    repro_zipfile_arc2 = rel_path / "repro_zipfile2.zip"
    with ReproducibleZipFile(repro_zipfile_arc2, "w") as zp:
        for path in sorted(dir_tree.glob("**/*")):
            zp.write(str(path))

    # Create second regular ZipFile archive for comparison after delay
    zipfile_arc2 = rel_path / "zipfile2.zip"
    with ZipFile(zipfile_arc2, "w") as zp:
        for path in sorted(dir_tree.glob("**/*")):
            zp.write(str(path))

    # All four archives should have identical content
    assert_archive_contents_equals(repro_zipfile_arc1, zipfile_arc1)
    assert_archive_contents_equals(repro_zipfile_arc1, repro_zipfile_arc2)
    assert_archive_contents_equals(repro_zipfile_arc1, zipfile_arc2)

    # ReproducibleZipFile hashes should match; ZipFile hashes should not
    assert hash_file(repro_zipfile_arc1) == hash_file(repro_zipfile_arc2)
    assert hash_file(zipfile_arc1) != hash_file(zipfile_arc2)


def test_write_single_file(base_path):
    """Writing the same file with different mtime produces the same hash."""
    data_file = file_factory(base_path)

    repro_zip1 = base_path / "repro_zip1.zip"
    with ReproducibleZipFile(repro_zip1, "w") as zp:
        zp.write(data_file)

    zip1 = base_path / "zip1.zip"
    with ZipFile(zip1, "w") as zp:
        zp.write(data_file)

    print(data_file.stat())
    sleep(2)
    data_file.touch()
    print(data_file.stat())

    repro_zip2 = base_path / "repro_zip2.zip"
    with ReproducibleZipFile(repro_zip2, "w") as zp:
        zp.write(data_file)

    zip2 = base_path / "zip2.zip"
    with ZipFile(zip2, "w") as zp:
        zp.write(data_file)

    # All four archives should have identical content
    assert_archive_contents_equals(repro_zip1, zip1)
    assert_archive_contents_equals(repro_zip1, repro_zip2)
    assert_archive_contents_equals(repro_zip1, zip2)

    # ReproducibleZipFile hashes should match; ZipFile hashes should not
    assert hash_file(repro_zip1) == hash_file(repro_zip2)
    assert hash_file(zip1) != hash_file(zip2)


def test_write_single_file_source_date_epoch(base_path, monkeypatch):
    """Writing the same file with different mtime with SOURCE_DATE_EPOCH set produces the
    same hash."""

    data_file = file_factory(base_path)

    arc_base = base_path / "base.zip"
    with ReproducibleZipFile(arc_base, "w") as zp:
        zp.write(data_file)

    monkeypatch.setenv("SOURCE_DATE_EPOCH", "1691732367")
    if tzset is not None:
        monkeypatch.setenv("TZ", "America/Chicago")
        tzset()

    # With SOURCE_DATE_EPOCH set
    arc_sde1 = base_path / "with_sde1.zip"
    with ReproducibleZipFile(arc_sde1, "w") as zp:
        zp.write(data_file)

    sleep(2)
    data_file.touch()
    if tzset is not None:
        # Set a different timezone to make sure it doesn't affect the set time
        monkeypatch.setenv("TZ", "America/Los_Angeles")
        tzset()

    arc_sde2 = base_path / "with_sde2.zip"
    with ReproducibleZipFile(arc_sde2, "w") as zp:
        zp.write(data_file)

    # All four archives should have identical content
    assert_archive_contents_equals(arc_base, arc_sde1)
    assert_archive_contents_equals(arc_base, arc_sde2)

    # Base archive hash should match neither, two archives with SOURCE_DATE_EPOCH should match
    assert hash_file(arc_base) != hash_file(arc_sde1)
    assert hash_file(arc_sde1) == hash_file(arc_sde2)


def test_write_single_file_file_mode_env_var(rel_path, monkeypatch):
    """REPRO_ZIPFILE_FILE_MODE environment variable works."""

    with umask(0o002):
        # Expect 664
        data_file = file_factory(rel_path)

    monkeypatch.setenv("REPRO_ZIPFILE_FILE_MODE", "600")  # rw-------

    arc_path = rel_path / "archive.zip"
    with ReproducibleZipFile(arc_path, "w") as zp:
        zp.write(data_file)

    with ZipFile(arc_path, "r") as zp:
        print(zp.infolist())
        mode = (zp.getinfo(data_file.name).external_attr >> 16) & 0o777

    assert mode == 0o600, (oct(mode), oct(0o600))


def test_write_single_file_string_paths(rel_path):
    """Writing the same file with different mtime produces the same hash, using string inputs
    instead of Path."""
    data_file = file_factory(rel_path)
    file_name = data_file.name
    assert isinstance(file_name, str)

    repro_zip1 = rel_path / "repro_zip1.zip"
    with ReproducibleZipFile(repro_zip1, "w") as zp:
        zp.write(file_name)

    zip1 = rel_path / "zip1.zip"
    with ZipFile(zip1, "w") as zp:
        zp.write(file_name)

    sleep(2)
    data_file.touch()

    repro_zip2 = rel_path / "repro_zip2.zip"
    with ReproducibleZipFile(repro_zip2, "w") as zp:
        zp.write(file_name)

    zip2 = rel_path / "zip2.zip"
    with ZipFile(zip2, "w") as zp:
        zp.write(file_name)

    # All four archives should have identical content
    assert_archive_contents_equals(repro_zip1, zip1)
    assert_archive_contents_equals(repro_zip1, repro_zip2)
    assert_archive_contents_equals(repro_zip1, zip2)

    # ReproducibleZipFile hashes should match; ZipFile hashes should not
    assert hash_file(repro_zip1) == hash_file(repro_zip2)
    assert hash_file(zip1) != hash_file(zip2)


def test_write_single_file_arcname(base_path):
    """Writing a single file with explicit arcname."""
    data_file = file_factory(base_path)

    repro_zip1 = base_path / "repro_zip1.zip"
    with ReproducibleZipFile(repro_zip1, "w") as zp:
        zp.write(data_file, arcname="lore.txt")

    zip1 = base_path / "zip1.zip"
    with ZipFile(zip1, "w") as zp:
        zp.write(data_file, arcname="lore.txt")

    sleep(2)
    data_file.touch()

    repro_zip2 = base_path / "repro_zip2.zip"
    with ReproducibleZipFile(repro_zip2, "w") as zp:
        zp.write(data_file, arcname="lore.txt")

    zip2 = base_path / "zip2.zip"
    with ZipFile(zip2, "w") as zp:
        zp.write(data_file, arcname="lore.txt")

    # All four archives should have identical content
    assert_archive_contents_equals(repro_zip1, zip1)
    assert_archive_contents_equals(repro_zip1, repro_zip2)
    assert_archive_contents_equals(repro_zip1, zip2)

    # ReproducibleZipFile hashes should match; ZipFile hashes should not
    assert hash_file(repro_zip1) == hash_file(repro_zip2)
    assert hash_file(zip1) != hash_file(zip2)


def test_write_single_dir_dir_mode_env_var(rel_path, monkeypatch):
    """REPRO_ZIPFILE_DIR_MODE environment variable works."""

    with umask(0o002):
        # Expect 775
        dir_path = rel_path / data_factory()
        dir_path.mkdir()

    monkeypatch.setenv("REPRO_ZIPFILE_DIR_MODE", "700")  # rwx------

    arc_path = rel_path / "archive.zip"
    with ReproducibleZipFile(arc_path, "w") as zp:
        zp.write(dir_path)

    with ZipFile(arc_path, "r") as zp:
        print(zp.infolist())
        mode = (zp.getinfo(dir_path.name + "/").external_attr >> 16) & 0o777

    assert mode == 0o700, (oct(mode), oct(0o700))


def test_writestr(tmp_path):
    """writestr works as expected"""
    data = data_factory()

    repro_zip1 = tmp_path / "repro_zip1.zip"
    with ReproducibleZipFile(repro_zip1, "w") as zp:
        zp.writestr("data.txt", data=data)

    zip1 = tmp_path / "zip1.zip"
    with ZipFile(zip1, "w") as zp:
        zp.writestr("data.txt", data=data)

    sleep(2)

    repro_zip2 = tmp_path / "repro_zip2.zip"
    with ReproducibleZipFile(repro_zip2, "w") as zp:
        zp.writestr("data.txt", data=data)

    zip2 = tmp_path / "zip2.zip"
    with ZipFile(zip2, "w") as zp:
        zp.writestr("data.txt", data=data)

    # All four archives should have identical content
    assert_archive_contents_equals(repro_zip1, zip1)
    assert_archive_contents_equals(repro_zip1, repro_zip2)
    assert_archive_contents_equals(repro_zip1, zip2)

    # ReproducibleZipFile hashes should match; ZipFile hashes should not
    assert hash_file(repro_zip1) == hash_file(repro_zip2)
    assert hash_file(zip1) != hash_file(zip2)


def test_writestr_zip_info(rel_path):
    """writestr with ZipInfo input"""
    data = data_factory()
    data_file = rel_path / "data.txt"
    data_file.touch()

    repro_zip1 = rel_path / "repro_zip1.zip"
    with ReproducibleZipFile(repro_zip1, "w") as zp:
        zp.writestr(ZipInfo.from_file(data_file), data=data)

    zip1 = rel_path / "zip1.zip"
    with ZipFile(zip1, "w") as zp:
        zp.writestr(ZipInfo.from_file(data_file), data=data)

    sleep(2)
    data_file.touch()

    repro_zip2 = rel_path / "repro_zip2.zip"
    with ReproducibleZipFile(repro_zip2, "w") as zp:
        zp.writestr(ZipInfo.from_file(data_file), data=data)

    zip2 = rel_path / "zip2.zip"
    with ZipFile(zip2, "w") as zp:
        zp.writestr(ZipInfo.from_file(data_file), data=data)

    # All four archives should have identical content
    assert_archive_contents_equals(repro_zip1, zip1)
    assert_archive_contents_equals(repro_zip1, repro_zip2)
    assert_archive_contents_equals(repro_zip1, zip2)

    # ReproducibleZipFile hashes should match; ZipFile hashes should not
    assert hash_file(repro_zip1) == hash_file(repro_zip2)
    assert hash_file(zip1) != hash_file(zip2)


def test_writestr_source_date_epoch(tmp_path, monkeypatch):
    """Test that using writestr with the same data at different times with SOURCE_DATE_EPOCH set
    produces the same hash."""

    data = data_factory()

    arc_base = tmp_path / "base.zip"
    with ReproducibleZipFile(arc_base, "w") as zp:
        zp.writestr("data.txt", data=data)

    sleep(2)

    monkeypatch.setenv("SOURCE_DATE_EPOCH", "1691732367")

    arc_sde1 = tmp_path / "sde1.zip"
    with ReproducibleZipFile(arc_sde1, "w") as zp:
        zp.writestr("data.txt", data=data)

    arc_sde2 = tmp_path / "sde2.zip"
    with ReproducibleZipFile(arc_sde2, "w") as zp:
        zp.writestr("data.txt", data=data)

    # All three archives should have identical content
    assert_archive_contents_equals(arc_base, arc_sde1)
    assert_archive_contents_equals(arc_base, arc_sde2)

    # Base archive hash should match neither; two with SOURCE_DATE_EPOCH should match
    assert hash_file(arc_base) != hash_file(arc_sde1)
    assert hash_file(arc_sde1) == hash_file(arc_sde2)


def test_mkdir(rel_path):
    """mkdir is a Python 3.11+ method that we've backported to ReproducibleZipFile for <3.11, so
    that we can use the 3.11 implementation of write. Other tests don't cover direct use with a
    string input. Test that this is the same as writing a directory with ZipFile.
    """
    arc_repro1 = rel_path / "repro1.zip"
    with ReproducibleZipFile(arc_repro1, "w") as zp:
        zp.mkdir("dir")

    sleep(2)

    # Hashes are the same in second copy
    arc_repro2 = rel_path / "repro2.zip"
    with ReproducibleZipFile(arc_repro2, "w") as zp:
        zp.mkdir("dir")

    assert hash_file(arc_repro1) == hash_file(arc_repro2)

    # Produces same contests as regular ZipFile
    dir_path = rel_path / "dir"
    dir_path.mkdir()
    arc_zip = rel_path / "zip.zip"
    with ZipFile(arc_zip, "w") as zp:
        zp.write(dir_path)

    assert_archive_contents_equals(arc_repro1, arc_zip)



from glob import glob
import subprocess
import sys

from typer.testing import CliRunner

from repro_zipfile import __version__ as repro_zipfile_version
from rpzip import __version__ as rpzip_version
from rpzip import app
from tests.utils import (
    assert_archive_contents_equals,
    dir_tree_factory,
    file_factory,
    remove_ansi_escape,
)

runner = CliRunner()


def test_zip_single_file(base_path):
    data_file = file_factory(base_path)

    rpzip_out = base_path / "rpzip.zip"
    rpzip_args = [str(rpzip_out), str(data_file)]
    rpzip_result = runner.invoke(app, rpzip_args)
    assert rpzip_result.exit_code == 0, rpzip_args

    zip_out = base_path / "zip.zip"
    zip_cmd = ["zip", str(zip_out), str(data_file)]
    zip_result = subprocess.run(zip_cmd)
    assert zip_result.returncode == 0, zip_cmd

    assert_archive_contents_equals(rpzip_out, zip_out)


def test_zip_directory(base_path):
    """Single directory, not recursive."""
    dir_tree = dir_tree_factory(base_path)

    rpzip_out = base_path / "rpzip.zip"
    rpzip_args = [str(rpzip_out), str(dir_tree)]
    rpzip_result = runner.invoke(app, rpzip_args)
    assert rpzip_result.exit_code == 0, rpzip_args

    zip_out = base_path / "zip.zip"
    zip_cmd = ["zip", str(zip_out), str(dir_tree)]
    zip_result = subprocess.run(zip_cmd)
    assert zip_result.returncode == 0, zip_cmd

    assert_archive_contents_equals(rpzip_out, zip_out)


def test_zip_directory_recursive(base_path):
    """Single input directory with recursive -r flag."""
    dir_tree = dir_tree_factory(base_path)

    rpzip_out = base_path / "rpzip.zip"
    rpzip_args = ["-r", str(rpzip_out), str(dir_tree)]
    rpzip_result = runner.invoke(app, rpzip_args)
    assert rpzip_result.exit_code == 0, rpzip_args

    zip_out = base_path / "zip.zip"
    zip_cmd = ["zip", "-r", str(zip_out), str(dir_tree)]
    zip_result = subprocess.run(zip_cmd)
    assert zip_result.returncode == 0, zip_cmd

    assert_archive_contents_equals(rpzip_out, zip_out)


def test_zip_multiple_recursive(base_path):
    """Mulitiple input files with recursive -r flag."""
    dir_tree = dir_tree_factory(base_path)

    rpzip_out = base_path / "rpzip.zip"
    rpzip_args = ["-r", str(rpzip_out)] + glob(str(dir_tree))
    rpzip_result = runner.invoke(app, rpzip_args)
    assert rpzip_result.exit_code == 0, rpzip_args

    zip_out = base_path / "zip.zip"
    zip_cmd = ["zip", "-r", str(zip_out)] + glob(str(dir_tree))
    zip_result = subprocess.run(zip_cmd)
    assert zip_result.returncode == 0, zip_cmd

    assert_archive_contents_equals(rpzip_out, zip_out)


def test_zip_no_suffix_adds_suffix(base_path):
    """Appropriately add .zip suffix if file does not have one."""
    data_file = file_factory(base_path)

    # Should add .zip to argument without suffix
    rpzip_out_expected = base_path / "rpzip.zip"
    rpzip_out_arg = rpzip_out_expected.with_suffix("")
    rpzip_args = [str(rpzip_out_arg), str(data_file)]
    rpzip_result = runner.invoke(app, rpzip_args)
    assert rpzip_result.exit_code == 0, rpzip_args
    assert rpzip_out_expected.exists(), (rpzip_args, list(base_path.iterdir()))

    # zip also adds .zip to argument another argument without suffix
    zip_out_expected = base_path / "zip.zip"
    zip_out_arg = zip_out_expected.with_suffix("")
    zip_cmd = ["zip", str(zip_out_arg), str(data_file)]
    zip_result = subprocess.run(zip_cmd)
    assert zip_result.returncode == 0, zip_cmd
    assert zip_out_expected.exists(), (zip_cmd, list(base_path.iterdir()))

    assert_archive_contents_equals(rpzip_out_expected, zip_out_expected)


def test_zip_existing_suffix_does_not_add_suffix(base_path):
    """Does not add .zip suffix if file already has one."""
    data_file = file_factory(base_path)

    # Should not add .zip to argument with existing suffix
    rpzip_out_expected = base_path / "rpzip.some_suffix"
    rpzip_args = [str(rpzip_out_expected), str(data_file)]
    rpzip_result = runner.invoke(app, rpzip_args)
    assert rpzip_result.exit_code == 0, rpzip_args
    assert rpzip_out_expected.exists(), (rpzip_args, list(base_path.iterdir()))

    # zip also does not add .zip to argument with existing suffix
    zip_out_expected = base_path / "zip.some_suffix"
    zip_cmd = ["zip", str(zip_out_expected), str(data_file)]
    zip_result = subprocess.run(zip_cmd)
    assert zip_result.returncode == 0, zip_cmd
    assert zip_out_expected.exists(), (zip_cmd, list(base_path.iterdir()))

    assert_archive_contents_equals(rpzip_out_expected, zip_out_expected)


def test_verbosity(rel_path):
    """Adjustment of verbosity with -v and -q."""
    data_file = file_factory(rel_path)
    rpzip_out = rel_path / "rpzip.zip"

    # Base case, should be INFO level
    rpzip_args = [str(rpzip_out), str(data_file)]
    rpzip_result = runner.invoke(app, rpzip_args)
    assert rpzip_result.exit_code == 0, rpzip_args
    assert "INFO" in rpzip_result.output
    assert "DEBUG" not in rpzip_result.output

    # With -v, should be DEBUG level
    rpzip_args = [str(rpzip_out), str(data_file), "-v"]
    rpzip_result = runner.invoke(app, rpzip_args)
    assert rpzip_result.exit_code == 0, rpzip_args
    assert "INFO" in rpzip_result.output
    assert "DEBUG" in rpzip_result.output

    # With -q, should be no output
    rpzip_args = [str(rpzip_out), str(data_file), "-q"]
    rpzip_result = runner.invoke(app, rpzip_args)
    assert rpzip_result.exit_code == 0, rpzip_args
    assert rpzip_result.output.strip() == ""


def test_version():
    """With --version flag."""
    result = runner.invoke(app, ["--version"])
    assert result.exit_code == 0
    output_lines = result.output.split("\n")
    assert output_lines[0].startswith("repro-zipfile ")
    assert output_lines[0].endswith(f"v{repro_zipfile_version}")
    assert output_lines[1].startswith("rpzip ")
    assert output_lines[1].endswith(f"v{rpzip_version}")


def test_python_dash_m_invocation():
    result = subprocess.run(
        [sys.executable, "-m", "rpzip", "--help"],
        capture_output=True,
        text=True,
        universal_newlines=True,
    )
    assert result.returncode == 0
    assert "Usage: python -m rpzip" in remove_ansi_escape(result.stdout)



"""Basic usage example of repro_zipfile."""

from pathlib import Path

from repro_zipfile import ReproducibleZipFile

example_dir = Path(__file__).parent

with ReproducibleZipFile(example_dir / "archive.zip", "w") as zp:
    # Use write to add a file to the archive
    zp.write(example_dir / "data.txt", arcname="data.txt")
    # Or writestr to write data to the archive
    zp.writestr("lore.txt", data="goodbye")



"""A demo script that shows repro_zipfile's reproducible output in contrast with the standard
library module zipfile's nonreproducible output.
"""

import hashlib
from pathlib import Path
from tempfile import TemporaryDirectory
from time import sleep
from zipfile import ZipFile

from repro_zipfile import ReproducibleZipFile

example_dir = Path(__file__).parent


def create_archive(zip_class, outfile: Path):
    with zip_class(outfile, "w") as zp:
        # Use write to add a file to the archive
        zp.write(example_dir / "data.txt", arcname="data.txt")
        # Or writestr to write data to the archive
        zp.writestr("lore.txt", data="goodbye")


with TemporaryDirectory() as tempdir_name:
    tempdir = Path(tempdir_name)

    cases = [
        (ZipFile, tempdir / "zipfile-1.zip"),
        (ZipFile, tempdir / "zipfile-2.zip"),
        (ReproducibleZipFile, tempdir / "repro_zipfile-1.zip"),
        (ReproducibleZipFile, tempdir / "repro_zipfile-2.zip"),
    ]

    for case in cases:
        create_archive(*case)
        sleep(2)

    for _, outfile in cases:
        print(outfile.name, hashlib.md5(outfile.read_bytes()).hexdigest())
